library debounce;

import 'dart:async';

class Debounce<S, T> extends StreamEventTransformer {

  final Duration _delay;

  const Debounce(this._delay);

  Stream<T> bind(Stream<S> stream) {
    var initial = new DateTime.now();
    var next    = initial.subtract(this._delay);
    return stream.where((S data) {
      var now = new DateTime.now();
      if (now.isAfter(next)) {
        next = now.add(this._delay);
        return true;
      } else {
        return false;
      }
    });
  }

}